library(tidyverse)
knitr::opts_chunk$set(dpi = 600, fig.width = 8, fig.height = 5)
config <- yaml::read_yaml('config.yml')



# baseline data (no table produced) ============================================
df_baseline <- read_rds("output/v1.1/data/df_baseline.rds") %>%
  mutate(
    InjuryHx.PupilsBaselineDerived = factor(InjuryHx.PupilsBaselineDerived,
                                            levels = 0:2),
    InjuryHx.GCSScoreBaselineDerived = case_when(
      InjuryHx.GCSScoreBaselineDerived <= 8  ~ "Severe",
      InjuryHx.GCSScoreBaselineDerived <= 12 ~ "Moderate",
      InjuryHx.GCSScoreBaselineDerived  > 12 ~ "Mild"
    ) %>% as.factor(),
    Labs.DLGlucosemmolL = exp(Labs.DLGlucosemmolL_log)
  ) %>%
  select(-Labs.DLGlucosemmolL_log) %>%
  left_join(
    read_rds(
      "data/v1.1/df_baseline_descriptive.rds"
    ) %>%
    transmute(
      gupi,
      Subject.PatientType = factor(
        Subject.PatientType,
        level = 1:3,
        labels = c("Emergency Room", "Admission to Hospital", "Intensive Care Unit")
      ),
      InjuryHx.InjCause = factor(InjuryHx.InjCause,
        level  = c(1:6, 88, 99),
        labels = c(
          "Road traffic incident",
          "Incidental fall",
          "Other non-intentional injury",
          "Violence/assault",
          "Act of mass violence",
          "Suicide attempt",
          "Unknown",
          "Other"
        ) %>%
        fct_recode(
          Other              = "Other non-intentional injury",
          Other              = "Suicide attempt",
          `Violence/assault` = "Act of mass violence"
        )
      ),
      InjuryHx.TotalISS
    ),
    by = "gupi"
  )

var_labels <- readr::read_csv("varlabels.csv")

get_label <- function(varname) var_labels$label[var_labels$varname == varname]
get_label <- Vectorize(get_label)

names(df_baseline) <- get_label(names(df_baseline))

n_pat <- nrow(df_baseline)

df_gose <- read_rds("output/v1.1/data/df_gose.rds")

n_gose_180 <- df_gose %>%
  filter(between(Outcomes.DerivedCompositeGOSEDaysPostInjury, 180 - 14, 180 + 14)) %>%
  group_by(gupi) %>%
  n_groups()

n_gose_pp <- df_gose %>%
  filter(between(Outcomes.DerivedCompositeGOSEDaysPostInjury, 5*30, 8*30)) %>%
  group_by(gupi) %>%
  n_groups()



# figure 1: gose longitudinal ==================================================
p_gose_times <- df_gose %>%
  ggplot(aes(Outcomes.DerivedCompositeGOSEDaysPostInjury / 30)) +
    geom_histogram(binwidth = 1) +
    theme_bw() +
    scale_x_continuous("Months post-injury",
      breaks = seq(0, 50)
    ) +
    scale_y_continuous("", breaks = seq(0, 2000, 200)) +
    theme(
      panel.grid.minor = element_blank(),
      panel.grid.major.x = element_blank()
    )

timepoint <- function(x) {
  abs(
    matrix(rep(x, 4), ncol = 4)
    - matrix(rep(c(14, 90, 180, 360), length(x)), nrow = length(x), byrow = TRUE)
  ) %>%
  apply(., 1, which.min) %>%
  factor(
    levels = 1:4,
    labels = c("2 weeks", "3 months", "6 months", "12 months")
  )
}

tmp <- df_gose %>%
  transmute(
    gupi,
    t    = timepoint(Outcomes.DerivedCompositeGOSEDaysPostInjury),
    GOSe = Outcomes.DerivedCompositeGOSE
  )

p1 <- tmp %>%
  ggplot(aes(GOSe)) +
    geom_bar(aes(fill = GOSe), width = .66, color = "black") +
    scale_fill_grey(start = .1, end = .9) +
    facet_wrap(~t, nrow = 1) +
    ylab("absolute count") +
    theme_bw() +
    theme(
      panel.grid.major.x = element_blank(),
      legend.position = "none"
    )

p2 <- tmp %>%
  group_by(t, GOSe) %>%
  summarize(n = n()) %>%
  group_by(t) %>%
  mutate(freq = n / sum(n)) %>%
  ggplot(aes(x = GOSe, y = freq)) +
    geom_bar(aes(fill = GOSe), width = .66, color = "black", stat = "identity") +
    scale_fill_grey(start = .1, end = .9) +
    facet_wrap(~t, nrow = 1) +
    ylab("relative frequency") +
    theme_bw() +
    theme(
      panel.grid.major.x  = element_blank(),
      legend.position = "none"
    )

p3 <- gridExtra::grid.arrange(
  p_gose_times,
  p1,
  p2,
  nrow    = 3,
  heights = c(1.5, 1, 1)
)

ggsave(filename = "gose_longitudinally.eps", plot = p3, width = 6, height = 6, colormodel = "cmyk")



# load model data ==============================================================
df_ground_truth <- c()

for (i in 1:config$folds) {

  df_ground_truth <- rbind(
    df_ground_truth,
    # note that GOSE values in test data are not subject to mi - loading the
    # first data set is sufficient!
    sprintf("output/v1.1/data/validation/df_test_mi_1_fold_%i.rds", i) %>%
      readRDS() %>%
      mutate(fold = i) %>%
      select(gupi, fold, Outcomes.DerivedCompositeGOSEDaysPostInjury, Outcomes.DerivedCompositeGOSE)
  )

}

modelnames <- lapply(
    list.dirs(
      "output/v1.1/data/validation/posteriors",
      recursive = FALSE
    ),
    basename
  ) %>%
  as.character()

df_model_posteriors <- c()

for (modelname in modelnames) {
  for (i in 1:config$mi_m) {
    for (j in 1:config$folds) {
      tryCatch(
        df_model_posteriors <- rbind(
          df_model_posteriors,
          sprintf("output/v1.1/data/validation/posteriors/%s/df_posterior_mi_%i_fold_%i.rds",
                  modelname, i, j) %>%
            readRDS() %>%
            mutate(imp = i, fold = j, model = modelname) %>%
            select(gupi, fold, imp, model, everything()) %>%
            filter(
              gupi %in% (df_ground_truth %>% filter(fold == j) %>% .[["gupi"]])
            )
        ),
        error = function(e) print(e)
      )
    }
  }
}

df_model_posteriors <- df_model_posteriors %>%
  mutate(
    model = factor(model,
      levels = c("locf", "mm_nb", "mm", "gp_nb", "gp", "msm", "msm_age"),
      labels = c("LOCF", "MM", "MM + cov", "GP", "GP + cov", "MSM", "MSM + age"),
    ) %>% as.character
  )

df_predictions <- df_model_posteriors %>%
  group_by(fold, model, gupi, t, GOSE) %>%
  summarise(probability = mean(probability)) %>%
  arrange(model, fold, gupi, t, GOSE) %>%
  ungroup() %>%
  filter(t == 180) %>%
  select(-t) %>%
  group_by(fold, model, gupi) %>%
  do(
    GOSE = .$GOSE,
    prediction = rep(which.max(.$probability) %>%
      ifelse(length(.) == 0, NA_integer_, .) %>%
      ifelse(length(.) > 1, round(mean(.)), .), 8),
    probability = .$probability
  ) %>%
  unnest %>%
  spread(GOSE, probability) %>%
  right_join(
    df_ground_truth %>%
      mutate(
        gupi = as.character(gupi),
        GOSE = as.integer(Outcomes.DerivedCompositeGOSE)
    ),
    by = c("gupi", "fold")
  )

idx <- df_predictions %>%
  filter(model == "LOCF", !complete.cases(.)) %>%
  .[["gupi"]]
compute_summary_measures <- function(df) {

  df %>%
    group_by(model, fold) %>%
    summarize(
      RMSE    = mean((GOSE - prediction)^2, na.rm = TRUE) %>% sqrt,
      MAE     = mean(abs(GOSE - prediction), na.rm = TRUE),
      bias    = mean(prediction, na.rm = TRUE) - mean(GOSE, na.rm = TRUE),
      `d-bias` = mean(prediction > GOSE, na.rm = TRUE) - mean(prediction < GOSE, na.rm = TRUE)
    ) %>%
    ungroup %>%
    gather(error, value, -model, -fold) %>%
    mutate(
      error = factor(error, c(
        "bias",
        "d-bias",
        "MAE",
        "RMSE"
      ))
    ) %>%
    group_by(model, error) %>%
    summarize(
      mean_error = mean(value),
      se_error   = sd(value) / sqrt(n()),
    )

}

df_plot <- rbind(
  compute_summary_measures(
      df_predictions %>%
        mutate(prediction = ifelse(model == "LOCF", NA, prediction))
    ) %>%
    mutate(label = "full test set"),
  compute_summary_measures(df_predictions %>% filter(!(gupi %in% idx))) %>%
    mutate(label = "LOCF subset")
) %>%
  ungroup



# figure 2: overall locf =======================================================
ggplot(df_plot, aes(x = model, y = mean_error)) +
  geom_hline(yintercept = 0, color = "black") +
  geom_point(aes(color = label), size = .8, position = position_dodge(.4)) +
  geom_errorbar(
    aes(ymin = mean_error - 1.96*se_error, ymax = mean_error + 1.96*se_error, color = label),
    width = .25, position = position_dodge(.3)
  ) +
  scale_color_grey("", start = .0, end = .5) +
  facet_wrap(~error, nrow = 1) +
  scale_y_continuous(name = "", breaks = seq(-2, 8, .25), limits = c(-.5, 1.25)) +
  scale_x_discrete("") +
  theme_bw() +
  theme(
    panel.grid.minor   = element_blank(),
    panel.grid.major.x = element_blank(),
    axis.text.x        = element_text(angle = 66, hjust = 1),
    legend.position    = "bottom"
  )

ggsave(filename = "overall_locf.eps", width = 6, height = 3.5, colormodel = "cmyk")



# figure 3: overall confusion ==================================================
plot_confusion_matrices <- function(df_predictions, models, nrow = 2, legendpos, scriptsize) {

  df_average_confusion_matrices <- df_predictions %>%
    select(-`1`, -`2`) %>%
    filter(model %in% models) %>%
    group_by(fold, model) %>%
    do(
      confusion_matrix = caret::confusionMatrix(
          data      = factor(.$prediction, levels = 3:8),
          reference = factor(.$GOSE, levels = 3:8)
        ) %>%
        as.matrix %>% as_tibble %>%
        mutate(`Predicted GOSE` = {row_number() + 2} %>% as.character) %>%
        gather(`Observed GOSE`, n, 1:6)
    ) %>%
    unnest %>%
    group_by(model, `Predicted GOSE`, `Observed GOSE`) %>%
    summarize(n = mean(n)) %>%
    ungroup %>%
    mutate(model = factor(model, models))

  # p_cnf_mtrx_raw <-
  df_average_confusion_matrices %>%
    ggplot(aes(`Observed GOSE`, `Predicted GOSE`)) +
      geom_text(
        aes(
          label    = sprintf("%3.f", n) %>% ifelse(. == "0.0", "", .),
          fontface = ifelse(`Observed GOSE` == `Predicted GOSE`, 'bold', 'plain')
        ),
        vjust = .5, size = scriptsize
      ) +
      geom_hline(yintercept = c(2, 4, 6) + .5, color = "black") +
      geom_vline(xintercept = c(2, 4, 6) + .5, color = "black") +
      coord_fixed(expand = FALSE, xlim = c(0.5, 6.5), ylim = c(0.5, 6.5)) +
      labs(x = "observed GOSe", y = "imputed GOSe", fill = "") +
      theme_bw() +
      theme(
        panel.grid      = element_blank(),
        legend.position = "none"
      ) +
      facet_wrap(~model, nrow = nrow) +
      ggtitle("Average confusion matrix across folds (absolute counts)")
}

plot_confusion_matrices(
  df_predictions %>%
    filter(!(gupi %in% idx)),
  c("MSM", "GP + cov", "MM", "LOCF"),
  nrow = 1,
  legendpos = "none",
  scriptsize = 2.5
)

ggsave(filename = "confusion_matrices_locf.eps", width = 6, height = 3, colormodel = "cmyk")



# figure 4: errors stratified locf =============================================
plot_summary_measures_cond <- function(df_predictions, models, label) {

  df_predictions %>%
    filter(model %in% models) %>%
    group_by(model, fold, GOSE) %>%
    summarize(
      RMSE = mean((GOSE - prediction)^2, na.rm = TRUE) %>% sqrt,
      MAE  = mean(abs(GOSE - prediction), na.rm = TRUE),
      bias = mean(prediction, na.rm = TRUE) - mean(GOSE, na.rm = TRUE),
      `d-bias` = mean(prediction > GOSE, na.rm = TRUE) - mean(prediction < GOSE, na.rm = TRUE)
    ) %>%
    gather(error, value, -model, -GOSE, -fold) %>%
    group_by(GOSE, model, error, fold) %>%
    summarize(
        mean_per_fold = mean(value) # overall mean
    ) %>%
    group_by(GOSE, model, error) %>%
    summarize(
        mean = mean(mean_per_fold), # overall mean
        se   = sd(mean_per_fold) / sqrt(n())
    ) %>%
    ungroup() %>%
    mutate(
      model = factor(model, models),
      error = factor(error, c(
        "bias",
        "d-bias",
        "MAE",
        "RMSE"
      ))
    ) %>%
  ggplot(aes(GOSE, color = model)) +
      geom_hline(yintercept = 0, color = "black") +
      geom_line(aes(y = mean)) +
      geom_point(aes(y = mean), size = .3,
        position = position_dodge(.3)) +
      geom_errorbar(aes(ymin = mean - 1.96*se, ymax = mean + 1.96*se),
        width = .33,
        size = .5,
        position = position_dodge(.2)
      ) +
      geom_point(aes(y = mean),
        position = position_dodge(.2)) +
      xlab("observed GOSe") +
      facet_wrap(~error, nrow = 1) +
      scale_y_continuous(name = "", breaks = seq(-2, 8, .5)) +
      theme_bw() +
      theme(
        panel.grid.minor   = element_blank(),
        panel.grid.major.x = element_blank(),
        legend.title       = element_blank(),
        legend.position    = "bottom"
      ) +
      ggtitle(label)

}

plot_summary_measures_cond(
  df_predictions %>% filter(!(gupi %in% idx)),
  c("MSM", "GP + cov", "MM", "LOCF"),
  "Summary measures by observed GOSe, LOCF subset"
)

ggsave(filename = "errors_stratified_locf.eps", width = 6, height = 3.5, colormodel = "cmyk")



# figure 5: confusion all ==================================================
plot_confusion_matrices(
  df_predictions,
  c("MSM", "GP + cov", "MM"),
  nrow = 1,
  legendpos = "none",
  scriptsize = 3
)

ggsave(filename = "confusion_matrices_all.eps", width = 6, height = 3, colormodel = "cmyk")



# figure 6: errors overall ==================================================
plot_summary_measures_cond(
  df_predictions %>% filter(!(gupi %in% idx)),
  c("MSM", "GP + cov", "MM"),
  "Summary measures by observed GOSe, full test set"
)

ggsave(filename = "imputation_error.eps", width = 6, height = 3.5, colormodel = "cmyk")



# figure 7:marginal gose per fold ==================================================
df_ground_truth %>%
    ggplot(aes(Outcomes.DerivedCompositeGOSE)) +
        geom_bar() +
        facet_wrap(~fold) +
        theme_bw() +
        ylab("") +
        xlab("GOSe") +
        theme(
            panel.grid.minor   = element_blank(),
            panel.grid.major.x = element_blank()
        )

ggsave(filename = "gose_marignal_per_fold.eps", width = 6, height = 3, colormodel = "cmyk")



# figure 8: diagram ==================================================
Q <- matrix(0, nrow = 8, ncol = 8)
for (i in 1:8) {
  for (j in 1:8) {
    if (i == j - 1 & i != 1) {
      Q[i, j] <- 1
    }
    if (i == j + 1) {
      Q[i, j] <- 1
    }
  }
}
Q[2:8, 1] <- 1 # allow instantaneous deaths
Q <- t(Q)
curvature <- Q / 5
curvature[1, ] <- 0

setEPS()
postscript("diagram.eps", width = 8, height = 4)
diagram::plotmat(
  Q,
  relsize = 1,
  shadow.size = 0,
  pos = c(1, 7),
  box.size = .033,
  arr.pos = .55,
  arr.length = .33,
  arr.type = "triangle",
  cex.txt = 0,
  curve = curvature
)
dev.off()

unlink('Rplots.pdf')
